import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.List;

/**
 * A Panel & Frame to display a Graph.
 * Note that this is implemented as a Panel that has a frame rather than a frame that has a panel, 
 * as the emphasis is on the display rather than the window
 * @author Joel Ross
 * @version Sp 2014
 */
public class GraphVisualization extends JPanel implements MouseListener, MouseMotionListener, ActionListener
{
	private final int BORDER = 20; //space to leave around the display
	private final Color NODE_COLOR = new Color(49,161,0); //a nice green color for the node
	
	private VisualGraph graph; //the graph to draw
	private VisualGraph.Node selected; //the node the user is dragging around

	private int graphWidth, graphHeight; //size of the graph (for easy access)
	private int panelWidth, panelHeight; //size of the display
	
	private JButton newRandomButton, newCompleteButton, layoutStartButton, layoutStopButton; //buttons for interaction
	private Timer clock;
	
	/**
	 * Constructs a new visualization window for the given graph
	 * @param graph The graph to visualize
	 */
	public GraphVisualization(VisualGraph graph)
	{
		this.graph = graph;
		this.graphWidth = graph.getWidth();
		this.graphHeight = graph.getHeight();
		this.panelWidth = graphWidth+BORDER*2;
		this.panelHeight = graphHeight+BORDER*2;
		this.selected = null;
		
		this.setPreferredSize(new Dimension(panelWidth,panelHeight));
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		
		//make a frame to hold the panel
		JFrame frame = new JFrame("Graph Layout");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.setLayout(new BorderLayout());
		frame.add(this,BorderLayout.CENTER);
		
		//layout for buttons
		newCompleteButton = new JButton("New Complete Graph");
		newCompleteButton.addActionListener(this);
		newRandomButton = new JButton("New Random Graph");
		newRandomButton.addActionListener(this);
		layoutStartButton = new JButton("Auto-Layout Graph");
		layoutStartButton.addActionListener(this);
		layoutStopButton = new JButton("Stop Auto-Layout");
		layoutStopButton.addActionListener(this);
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(2,2));
		buttonPanel.add(newCompleteButton);
		buttonPanel.add(newRandomButton);
		buttonPanel.add(layoutStartButton);
		buttonPanel.add(layoutStopButton);
		frame.add(buttonPanel, BorderLayout.SOUTH);
		
		//timer to do animation; 60fps
		clock = new Timer(15,this);

		frame.pack();
		frame.setVisible(true);
	}
	
	//repaint the panel -- draws the graph
	public void paintComponent(Graphics g)
	{
		Graphics2D g2d = (Graphics2D)g;

		//clear the screen
		g2d.setColor(Color.WHITE);
		g2d.fillRect(0, 0, panelWidth, panelHeight);
		g2d.setStroke(new BasicStroke(2));
		
		//draw the edges
		g2d.setPaint(Color.GRAY);
		List<VisualGraph.Edge> edges = graph.getEdges();
		for(VisualGraph.Edge e : edges)
		{
			g2d.drawLine(BORDER+e.start().x(), BORDER+e.start().y(), BORDER+e.end().x(), BORDER+e.end().y()); //offset positions
		}
		
		//draw the nodes
		List<VisualGraph.Node> nodes = graph.getNodes();
		for(VisualGraph.Node n : nodes)
		{
			int x = n.x();
			int y = n.y();
			int r = n.radius();
			g2d.setPaint(NODE_COLOR);
			g2d.fillOval(BORDER+x-r, BORDER+y-r, r+r, r+r); //offset positions
			g2d.setPaint(Color.WHITE);
			g2d.drawOval(BORDER+x-r, BORDER+y-r, r+r, r+r);
		}
		
	}
	
	//Respond to action events
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == clock) //repaint on the clock
		{
			graph.layout(selected);
			repaint();
		}
		else if(e.getSource() == newCompleteButton) //make a new complete graph (note this replaces the current graph)
		{
			clock.stop();
			String msg = "";
			do //prompt for inputs; looping while invalid
			{
				try{
					String result = JOptionPane.showInputDialog(msg+"How many nodes?");
					if(result == null)
						return;
					int nodes = Integer.parseInt(result);
					graph = new VisualGraph(graphWidth, graphHeight, nodes);
					msg = "";
				}catch(NumberFormatException nfe){
					msg = "Invalid entry. ";
				}
			} while(!msg.equals(""));
			repaint();			
		}
		else if(e.getSource() == newRandomButton) //make a new random graph (note this replaces the current graph)
		{
			clock.stop();
			String msg = "";
			do //prompt for inputs; looping while invalid
			{
				try{
					String result = JOptionPane.showInputDialog(msg+"How many nodes?");
					if(result == null)
						return;
					int nodes = Integer.parseInt(result);
					result = JOptionPane.showInputDialog("How many edges?");
					if(result == null)
						return;
					int edges = Integer.parseInt(result);
					graph = new VisualGraph(graphWidth, graphHeight, nodes, edges);
					msg = "";
				}catch(NumberFormatException nfe){
					msg = "Invalid entry. ";
				}
			} while(!msg.equals(""));
			repaint();
		}
		else if(e.getSource() == layoutStartButton) //start the animation
		{
			clock.restart();
		}
		else if(e.getSource() == layoutStopButton) //stop the animation
		{
			clock.stop();
		}
	}

	//When the mouse is pressed, grab the selected node
	public void mousePressed(MouseEvent e)
	{
		int mx = e.getX()-BORDER; //mouse location in graph (accounting for offset)
		int my = e.getY()-BORDER;
		List<VisualGraph.Node> nodes = graph.getNodes();
		for(int i=nodes.size()-1; i>=0; i--) //start from the back, that we get the last-painted nodes first
		{
			VisualGraph.Node n = nodes.get(i);
			double dist = Math.sqrt((mx-n.x())*(mx-n.x())+(my-n.y())*(my-n.y())); //distance from center
			if(dist < n.radius()) //if within circle
			{
				selected = n;
				return;
			}
		}
	}
	
	//when the mouse is released, we are not moving the node
	public void mouseReleased(MouseEvent e)
	{
		selected = null;
	}

	//when the mouse is dragged, update the position of the selected node
	public void mouseDragged(MouseEvent e)
	{
		if(selected != null)
		{
			int mx = e.getX()-BORDER; //mouse location in graph (accounting for offset)
			int my = e.getY()-BORDER;

			if(mx >= 0 && mx <= graphWidth)
				selected.setX(mx);
			if(my >= 0 && my <= graphHeight)
				selected.setY(my);
			repaint();
		}
	}

	//mouse methods that are not used
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mouseMoved(MouseEvent e){}
	public void mouseClicked(MouseEvent e){}


	//main method for testing
	public static void main(String[] args)
	{
		//tests with a 500x500 graph with 8 nodes
		new GraphVisualization(new VisualGraph(500,500, 8));
	}
}
