import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

/**
 * A graph that can be drawn visually.
 */
public class VisualGraph
{
	private static final long RANDOM_SEED = 0; //a "seed" for the random number generator. A value of 0 will result in truly random graphs, 
												  //any other value will produce consistently random graphs
	
	private static final int NODE_SIZE = 7; //default size of each node

	/**
	 * An inner class to represent a node in this graph.
	 */
	public static class Node
	{
		private Point2D.Double pos; //position of the node
		private int radius; //radius of the node

		public Node(double x, double y, int radius) {
			this.pos = new Point2D.Double(x, y);
			this.radius = radius;
		}

		public Node(int x, int y) {
			this(x,y,VisualGraph.NODE_SIZE);
		}
		
		/*******
		 * Getters and setters. Work with integer-precision values (for easy drawing)
		 *********/

		public int x() { 
			return (int)pos.x; 
		}
		
		public int y() {
			return (int)pos.y;
		}
		
		public int radius() {
			return radius;
		}
		
		public void setX(int x) {
			this.pos.x = x;
		}
		
		public void setY(int y){
			this.pos.y = y;
		}
	}

	/**
	 * An inner class representing an Edge in this graph.
	 */
	public static class Edge
	{
		private Node u; //the nodes the edge connects
		private Node v;

		public Edge(Node u, Node v) {
			this.u = u;
			this.v = v;
		}
		
		//the "first" node in the edge
		public Node start() {
			return u;
		}
		
		//the "second" node in the edge
		public Node end() {
			return v;
		}
	}


	/*********************
	 * Instance variables
	 ***********************/
	
	private ArrayList<Node> nodes; //the nodes in the graph
	private ArrayList<Edge> edges; //the edges in the graph

	private int width, height; //size of the graph
	
	private Random rand; //random number generator, for making graphs

	
	/**
	 * Layout the nodes neatly. The given excluded node should not be modified (i.e., it is being modified by the user)
	 * @param exclude one node to exclude from the automatic layout; null if all nodes should be considered
	 */
	public void layout(Node exclude)
	{
		//TODO FILL THIS IN!!

		
		
		
		
		
		
		
		
	}
	

	
	
	
	
	
	
	
	
	
	
	
	//Constructors
	
	/**
	 * Creates a new undirected graph with the given number of nodes. THe graph is <i>complete</i>---it has all possible edges between the given nodes
	 * @param width The width of the displayed graph
	 * @param height The height of the displayed graph
	 * @param numNodes The number of nodes in the graph. The graph will have edges between all pairs of nodes.
	 */
	public VisualGraph(int width, int height, int numNodes)
	{
		this.width = width;
		this.height = height;

		if(RANDOM_SEED > 0)
			rand = new Random(RANDOM_SEED);
		else
			rand = new Random();
		
		nodes = new ArrayList<Node>();
		edges = new ArrayList<Edge>();

		for(int i=0; i<numNodes; i++) //make the nodes
			nodes.add(new Node(rand.nextInt(width), rand.nextInt(height), NODE_SIZE));

		for(int i=0; i<numNodes; i++) //make the edges: one between each pair
		{
			for(int j=i+1; j<numNodes; j++)
			{
				edges.add(new Edge(nodes.get(i), nodes.get(j)));
			}
		}
	}
	
	/**
	 * Creates a new undirected graph with the given number of nodes and edges. Edges are randomly chosen from the created nodes.
	 * @param width The width of the displayed graph
	 * @param height The height of the displayed graph
	 * @param numNodes The number of nodes in the graph.
	 * @param numEdges The number of edges to include. A number <= 0 will produce no edges, and a number greater numNodes!/2 will result in a complete graph
	 */
	public VisualGraph(int width, int height, int numNodes, int numEdges)
	{
		this(width, height, numNodes); //make the complete graph
	
		//then remove the extra edges
		while(edges.size() > 0 && edges.size() > numEdges) //while too many edges
			edges.remove(rand.nextInt(edges.size())); //remove a random edge
	}
		

	//Other methods (used with the visualization)
	
	/**
	 * Returns an unmodifiable list of the nodes.
	 * @return The nodes in this graph.
	 */
	public List<Node> getNodes()
	{
		return Collections.unmodifiableList(nodes); //can make unmodifiable?
	}

	/**
	 * Returns an unmodifiable list of the edges.
	 * @return The edges in this graph.
	 */
	public List<Edge> getEdges()
	{
		return Collections.unmodifiableList(edges); //can make unmodifiable?
	}

	/**
	 * Returns the width of the graph (in pixels)
	 * @return the width of the graph
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * Returns the height of the graph (in pixels)
	 * @return the height of the graph
	 */
	public int getHeight()
	{
		return height;
	}
	

}
